@@ -1,5 +1,12 @@
-import functools
+__all__ = [
+    'DISTILLERS',
+    'BaseDistiller',
+    'DistillableProto',
+    'build_metaclass',
+]
+
 from typing import (
+    Any,
     Callable,
     Dict,
     Generator,
@@ -7,34 +14,39 @@ from typing import (
     List,
     Optional,
     Protocol,
+    Tuple,
     Type,
     TypeVar,
     cast,
-    no_type_check,
 )
 
 import torch
 import torch.nn as nn
-import warnings
-from mmcv.runner import BaseModule
 
-from ..base import Message, Workflow, WorkflowConfig, inc_iter, init_iter
+from ..base import (
+    Message,
+    Module,
+    ModuleList,
+    Registry,
+    Workflow,
+    WorkflowConfig,
+    transfer_weights,
+)
 from ..hooks import BaseHook
-from ..utils import ModelLoader
 
 T = TypeVar('T', bound='BaseDistiller')
 
 
-class BaseDistiller(BaseModule):
+class BaseDistiller(Module):
     DEBUG_PREFIX = '_debug_'
 
     @staticmethod
     def workflow_to_module(workflow: Workflow) -> nn.Module:
-        module = nn.ModuleList()
+        module = ModuleList()
         for job in workflow:
             step = job.step
             if not isinstance(step, nn.Module):
-                step = nn.ModuleList(step)
+                step = ModuleList(step)
             module.append(step)
         return module
 
@@ -45,10 +57,9 @@ class BaseDistiller(BaseModule):
         hooks: Dict[int, WorkflowConfig],
         adapts: WorkflowConfig,
         losses: WorkflowConfig,
-        iter_: int = 0,
         weight_transfer: Optional[Dict[str, str]] = None,
     ):
-        BaseModule.__init__(self)
+        Module.__init__(self)
         self._models = models
 
         self._hookflows: Dict[int, Workflow] = {  # yapf: disable
@@ -58,9 +69,8 @@ class BaseDistiller(BaseModule):
         self._adaptflow = Workflow.build('adapts', adapts)
         self._lossflow = Workflow.build('losses', losses)
 
-        init_iter(iter_)
         if weight_transfer is not None:
-            ModelLoader.load_state_dicts(self, weight_transfer)
+            transfer_weights(self, weight_transfer)
 
         for i, workflow in self._hookflows.items():
             for job in workflow:
@@ -115,80 +125,83 @@ class BaseDistiller(BaseModule):
         self,
         custom_tensors: Optional[Dict[str, torch.Tensor]] = None,
         debug: bool = False,
-        updated: bool = False,  # TODO: remove
     ) -> Dict[str, torch.Tensor]:
-        if not updated:
-            warnings.warn("needs to update")
-            self.track_tensors()
-
         tensors = self.tensors()
         if custom_tensors is not None:
             tensors.update(custom_tensors)
         self._adaptflow(tensors)
         losses = self._lossflow(tensors.copy())
 
-        if not updated:
-            inc_iter()
-            self.reset()
-
         if debug:
-            tensors = {self.DEBUG_PREFIX + k: v for k, v in tensors.items()}
-            return {**losses, **tensors}
+            losses.update({
+                self.DEBUG_PREFIX + k: v
+                for k, v in tensors.items()
+            })
         return losses
 
 
-class DistillerProto(Protocol):
-
-    def __init__(self, student: nn.Module, *args, **kwargs) -> None:
-        pass
+DISTILLERS: Registry[BaseDistiller] = Registry(
+    'distillers',
+    base=BaseDistiller,
+)
 
 
-DistillerType = TypeVar('DistillerType', bound=DistillerProto)
+class DistillableProto(Protocol):
+    _distiller: BaseDistiller
+
+    @property
+    def distiller(self) -> BaseDistiller:
+        ...
+
+    @property
+    def sync_apply(self) -> bool:
+        ...
 
 
-class WrapperProto(Protocol[DistillerType]):
-    _distiller: DistillerType
+def build_metaclass(
+    distiller_cls: Type[BaseDistiller],
+    supermetaclass: Type[type] = type,
+) -> type:
 
+    class MetaClass(supermetaclass):  # type: ignore[valid-type, misc]
+        NAMESPACE = dict(
+            distiller=property(
+                lambda x: cast(DistillableProto, x)._distiller,
+            ),
+            sync_apply=property(lambda _: False),
+        )
 
-WrapperType = TypeVar('WrapperType', bound=WrapperProto)
+        def __new__(
+            meta_cls,
+            cls: str,
+            bases: Tuple[type, ...],
+            namespace: Dict[str, Any],
+            **kwargs: Any,
+        ) -> 'MetaClass':
+            assert len(namespace.keys() & meta_cls.NAMESPACE.keys()) == 0
+            namespace.update(meta_cls.NAMESPACE)
+            return super().__new__(
+                meta_cls,
+                cls,
+                bases,
+                namespace,
+                **kwargs,
+            )
 
-WrappedType = TypeVar('WrappedType')
+        def __call__(
+            cls,
+            *args,
+            **kwargs,
+        ) -> DistillableProto:
+            if 'distiller' not in kwargs:
+                raise RuntimeError('`distiller` is required')
+            distiller: Dict[str, Any] = kwargs.pop('distiller')
+            obj = super().__call__(*args, **kwargs)
+            obj = cast(DistillableProto, obj)
+            obj._distiller = distiller_cls(  # type: ignore[call-arg]
+                student=obj,
+                **distiller,
+            )
+            return obj
 
-
-class DecoratorMixin:
-
-    @classmethod
-    def wrap(
-        cls: Type[DistillerType],
-    ) -> Callable[[Type[WrappedType]], Type[WrappedType]]:
-
-        @no_type_check
-        def wrapper(wrapped_cls):
-
-            class WrapperMeta(wrapped_cls.__class__):
-
-                def __call__(
-                    wrapper_cls,
-                    *args,
-                    distiller,
-                    **kwargs,
-                ) -> WrapperType:
-                    obj: WrapperType = super().__call__(*args, **kwargs)
-                    obj._distiller = cls(obj, **distiller)
-                    return obj
-
-            @functools.wraps(wrapped_cls, updated=())
-            class WrapperClass(wrapped_cls, metaclass=WrapperMeta):
-                _distiller: DistillerType
-
-                @property
-                def distiller(self):
-                    return self._distiller
-
-                @property
-                def sync_apply(self) -> bool:
-                    return False
-
-            return WrapperClass
-
-        return wrapper
+    return MetaClass
