@@ -1,14 +1,17 @@
+__all__ = [
+    'SelfDistiller',
+]
+
 from typing import Dict, Optional
 
 import torch.nn as nn
 
 from ..base import WorkflowConfig
-from .base import BaseDistiller, DecoratorMixin
-from .builder import DISTILLERS
+from .base import DISTILLERS, BaseDistiller
 
 
 @DISTILLERS.register_module()
-class SelfDistiller(DecoratorMixin, BaseDistiller):
+class SelfDistiller(BaseDistiller):
 
     def __init__(
         self,
