@@ -1,28 +1,26 @@
-import numbers
-from typing import Any, Literal, Optional, Union, cast
-
-import torch
-import torch.nn as nn
-from mmcv import ConfigDict
-from mmcv.runner import BaseModule
-
-from ..base import STEPS, Registry
-from ..schedulers import SCHEDULERS
-
 __all__ = [
     'BaseLoss',
     'LOSSES',
 ]
 
+import numbers
+from typing import Any, Literal, Mapping, Optional, Union, cast
+
+import torch
+import torch.nn as nn
+
+from ..base import STEPS, Module, Registry
+from ..schedulers import SCHEDULERS
+
 Reduction = Literal['none', 'mean', 'sum', 'prod']
 
 
-class BaseLoss(BaseModule):
+class BaseLoss(Module):
 
     def __init__(
         self,
         reduction: Reduction = 'mean',
-        weight: Union[numbers.Real, ConfigDict] = 1.0,
+        weight: Union[numbers.Real, Mapping] = 1.0,  # type: ignore[assignment]
         bound: Optional[numbers.Real] = None,
         **kwargs,
     ):
@@ -36,7 +34,9 @@ class BaseLoss(BaseModule):
         self._weight = weight
         self._threshold = None if bound is None else bound / weight
 
-        self.register_forward_hook(self._forward_hook)
+        self.register_forward_hook(
+            self._forward_hook,  # type: ignore[arg-type]
+        )
 
     @property
     def reduction(self) -> Reduction:
@@ -78,8 +78,8 @@ class BaseLoss(BaseModule):
         return output
 
 
-LOSSES: Registry[nn.Module] = Registry(
+LOSSES: Registry[BaseLoss] = Registry(
     'losses',
     parent=STEPS,
-    base=nn.Module,
+    base=BaseLoss,
 )
